import requests
import logging

from protatoquests.proxy import download_proxies, get_cached_proxies

logger = logging.getLogger(__name__)

def get(url, **kwargs):
    return request("GET", url, **kwargs)


def post(url, **kwargs):
    return request("POST", url, **kwargs)

def put(url, **kwargs):
    return request("PUT", url, **kwargs)


def delete(url, **kwargs):
    return request("DELETE", url, **kwargs)


def patch(url, **kwargs):
    return request("PATCH", url, **kwargs)


def request(method, url, **kwargs):
    return _request_with_proxy(False, method, url, **kwargs)


def _request_with_proxy(ssl_verify, method, url, **kwargs):
    proxies = get_cached_proxies()
    for proxy in proxies:
        logger.debug(f"Using proxy: {proxy}")
        try:
            args = {**kwargs, "proxies": {"https": "socks5://" + proxy}}
            if not ssl_verify:
                args["verify"] = False
            response = requests.request(method, url, **args)
        except Exception as e:
            logger.debug(f"Proxy {proxy} failed: {e}")
            continue
        if response.status_code < 400:
            logger.debug(f"Proxy {proxy} success")
            return response
        else:
            logger.debug(f"Proxy {proxy} failed: {response.status_code}: {response.text}")

    logger.warning("Failed to execute request with proxies, falling back to not using proxy")
    return requests.request(method, url, **kwargs)



import base64
import logging
import time
from functools import lru_cache

import requests

logger = logging.getLogger(__name__)

DEFAULT_CACHE_TTL = 300
cache_ttl_seconds = DEFAULT_CACHE_TTL

def set_cache_ttl(ttl: int):
    global cache_ttl_seconds
    cache_ttl_seconds = ttl

def get_cached_proxies():
    global cache_ttl_seconds
    cache_hash = round(time.time() / cache_ttl_seconds)
    return download_proxies(cache_hash=cache_hash)

@lru_cache
def download_proxies(cache_hash: int):
    response = requests.get("https://advanced.name/freeproxy?type=socks5")
    if response.status_code != 200:
        logger.error(f"Unable to scrape proxies: {response.status_code} {response.text})")
        return []
    from bs4 import BeautifulSoup
    parsed_html = BeautifulSoup(response.text)
    table = parsed_html.body.find("table", id="table_proxies")
    proxies = []
    for row in table.find_all("tr"):
        cols = row.find_all("td")
        if cols:
            ip = base64.b64decode(cols[1].get("data-ip")).decode("utf-8")
            port = base64.b64decode(cols[2].get("data-port")).decode("utf-8")
            proxies.append(f"{ip}:{port}")
    return proxies
